<?php
namespace App\Factory;

use App\Exception\ParamsException as ParamsException;

class OrderFactory {
    
    private $connection;
    private $util;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util) {
        $this->connection = $connection;
        $this->util       = $util;
    }
    
    public function getFilial($subsidiaryCode){
        $sql = "
                SELECT NMFILIAL 
                    FROM FILIAL
                WHERE CDFILIAL = :CDFILIAL
        ";
        
        $params = [
                ':CDFILIAL' => $subsidiaryCode
            ];
        $result = $this->connection->fetchAssoc($sql, $params);
        
        return $result['NMFILIAL'];
    }
    
    public function getSales($subsidiaryCode, $dtInicial, $dtFinal){
        $sql = "
                SELECT V.CDFILIAL,V.NRINSJURIEST, V.CDCAIXA, V.NRSEQVENDA, TO_CHAR(V.DTVENDA, 'dd-mm-yyyy hh24:mi:ss') AS DTVENDA,
                       TO_CHAR(V.DTENTRVENDA, 'dd-mm-yyyy') AS DTENTRVENDA, TO_CHAR(V.DTEMISSAONFCE, 'dd-mm-yyyy hh24:mi:ss') AS DTEMISSAONFCE,
                       V.IDSITUVENDA, V.IDSTATUSNFCE, V.NRACESSONFCE, V.NRACESSOCANC, V.NRNOTAFISCALCE, V.CDSERIENFCE, V.IDCANCMANUAL,
                       V.SGESTADO, V.NRLANCTONFCE, V.DSOBSSTATUSNFCE, V.DSOBSCANCUP, V.NRPROTOCOLONFCE, NRPROTOCOLOCANC, V.VRTOTVENDA, V.VRTXSEVENDA
                FROM VENDA V
                    WHERE V.CDFILIAL = :CDFILIAL
                        AND V.IDTPAMBNFCE = '2'
                        AND V.DTENTRVENDA BETWEEN TO_DATE(:DATA_INI,'dd/mm/yyyy') AND TO_DATE(:DATA_FIM,'dd/mm/yyyy')
                        AND NRSEQVENDA IS NOT NULL
                        AND NRNOTAFISCALCE IS NOT NULL
                ORDER BY DTVENDA, CDCAIXA    
                
        ";
        
        
        $params = [
            ':CDFILIAL' => $subsidiaryCode,
            ':DATA_INI' => $dtInicial,
            ':DATA_FIM' => $dtFinal
            ];
            
        
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getVrTotalVenda($subsidiaryCode, $cdCaixa , $nrSeqVenda){
        
        $sql = "
                SELECT SUM(SUM((QTPRODVEND*VRUNITVEND)-NVL(VRDESITVEND,0)+NVL(VRACRITVEND,0))) AS VRTOTAL
                FROM ITEMVENDA 
                    WHERE CDFILIAL = :CDFILIAL
                    AND CDCAIXA    = :CDCAIXA
                    AND NRSEQVENDA = :NRSEQVENDA
                    
                    GROUP BY NRSEQVENDA 
                ";
                
        $params = [
                ':CDFILIAL'   => $subsidiaryCode,
                ':CDCAIXA'    => $cdCaixa,
                ':NRSEQVENDA' => $nrSeqVenda
                ];
                
        return $this->connection->fetchAssoc($sql, $params);
    }
    
     public function getTotalAcreDesc($subsidiaryCode, $cdCaixa , $nrSeqVenda){
        
        $sql = "
                SELECT SUM(VRDESITVEND) AS VRDESITVEND,
                       SUM(VRACRITVEND) AS VRACRITVEND
                FROM ITEMVENDA
                WHERE CDFILIAL = :CDFILIAL
                    AND CDCAIXA    = :CDCAIXA
                    AND NRSEQVENDA = :NRSEQVENDA
                ";
                
        $params = [
                ':CDFILIAL'   => $subsidiaryCode,
                ':CDCAIXA'    => $cdCaixa,
                ':NRSEQVENDA' => $nrSeqVenda
                ];
                
        $result = $this->connection->fetchAssoc($sql, $params);
        
        $total = [
                'acrescimo' => isset($result['VRACRITVEND']) ? $result['VRACRITVEND'] : 0 ,
                'desconto'  => isset($result['VRDESITVEND']) ? $result['VRDESITVEND'] : 0 
            ];

        return $total;
    }
    
    public function getTotalImpostos($subsidiaryCode, $cdCaixa , $nrSeqVenda){
        
        $sql = "
                SELECT SUM(VRIMPPIS) AS VRIMPPIS,
                       SUM(VRIMPCOFINS) AS VRIMPCOFINS, 
                       SUM(VRIMPOPRODIT) AS VRIMPOPRODIT
                FROM ITVENDAIMPOS
                WHERE CDFILIAL = :CDFILIAL
                    AND CDCAIXA    = :CDCAIXA
                    AND NRSEQVENDA = :NRSEQVENDA
                ";
                
        $params = [
                ':CDFILIAL'   => $subsidiaryCode,
                ':CDCAIXA'    => $cdCaixa,
                ':NRSEQVENDA' => $nrSeqVenda
                ];
        
        $result = $this->connection->fetchAssoc($sql, $params);
        
        $total = [
                'vrPIS'    => isset($result['VRIMPPIS']) ? $result['VRIMPPIS'] : 0,
                'vrCOFINS' => isset($result['VRIMPCOFINS']) ? $result['VRIMPCOFINS'] : 0,
                'vrICMS'   => isset($result['VRIMPOPRODIT']) ? $result['VRIMPOPRODIT'] : 0
            ];
        
        return $total;
    }
    
    public function getClosingType($subsidiaryCode){
        $sql = "
                SELECT IDTPAPUFIL FROM PARAVEND
                WHERE CDFILIAL = :CDFILIAL
        ";
        $params = [
                ':CDFILIAL' => $subsidiaryCode
            ];
            
        return $this->connection->fetchAssoc($sql, $params);
        
    }
    
    public function getTipoSangria($subsidiaryCode){
        $sql = "
                SELECT CDTPSANGRIA 
                    FROM TIPOSANGRIA
                WHERE CDFILIAL = :CDFILIAL
                    AND IDSANGRIA = 'F'
        ";
        $params = [
                ':CDFILIAL' => $subsidiaryCode
            ];
            
        return $this->connection->fetchAssoc($sql, $params);
        
    }
    
    public function closingPosBySangria($subsidiaryCode, $dtInicial, $dtFinal, $cdTpSangria){
        $sql = "
                SELECT TO_DATE(DTABERCAIX,'DD/MM/YYYY hh24:mi:ss') AS DTABERCAIX, CDTIPORECE, SUM(VRMOVIVEND) AS VRMOVIVEND
                    FROM MOVCAIXA
                 WHERE CDFILIAL = :CDFILIAL
                        AND DTABERCAIX BETWEEN TO_DATE(:DATA_INI, 'DD/MM/YYYY hh24:mi:ss') AND TO_DATE(:DATA_FIM, 'DD/MM/YYYY hh24:mi:ss')
                        AND IDTIPOMOVIVE = 'G'
                        AND CDTPSANGRIA = :CDTPSANGRIA
                GROUP BY DTABERCAIX, CDTIPORECE
        ";
        $params = [
                    ':CDFILIAL'    => $subsidiaryCode,
                    ':DATA_INI'    => $dtInicial,
                    ':DATA_FIM'    => $dtFinal . ' ' . '23:59:59',
                    ':CDTPSANGRIA' => $cdTpSangria
            ];
            

        return $this->connection->fetchAll($sql, $params);
    }
    
    public function closingPosByVenda($subsidiaryCode, $dtInicial, $dtFinal){
        $sql = "
                SELECT TO_DATE(DTABERCAIX,'DD/MM/YYYY hh24:mi:ss') AS DTABERCAIX,
                        CDTIPORECE, SUM(CASE WHEN IDTIPOMOVIVE = 'E' 
                                     THEN VRMOVIVEND 
                                     ELSE - VRMOVIVEND END ) AS VRMOVIVEND
                   FROM MOVCAIXA
                     WHERE CDFILIAL = :CDFILIAL
                        AND DTABERCAIX BETWEEN TO_DATE(:DATA_INI, 'DD/MM/YYYY hh24:mi:ss') AND TO_DATE(:DATA_FIM, 'DD/MM/YYYY hh24:mi:ss')
                        AND IDTIPOMOVIVE IN ('E','S')
                    GROUP BY DTABERCAIX, CDTIPORECE
                ORDER BY DTABERCAIX, CDTIPORECE
        ";

        $params = [
                    ':CDFILIAL'    => $subsidiaryCode,
                    ':DATA_INI'    => $dtInicial,
                    ':DATA_FIM'    => $dtFinal . ' ' . '23:59:59'
            ];
        
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getPaymentInfo($cdTipoRece){
        $sql = "
                SELECT TR.CDTIPORECE, TR.NMTIPORECE, TR.IDTIPORECE, RI.CDTPRECEINTE
                    FROM TIPORECE TR
                  LEFT JOIN (SELECT TP.CDTPRECEINTE, TP.CDTIPORECE    
                        FROM TIPORECEINTE TP
                      JOIN INTEGRACAO INTE
                         ON TP.CDINTEGRACAO = INTE.CDINTEGRACAO
                      WHERE INTE.IDINTEGRACAO = 'DTS'
                        AND TP.CDTIPORECE = :CDTIPORECE) RI 
                    ON TR.CDTIPORECE = RI.CDTIPORECE
                WHERE TR.CDTIPORECE = :CDTIPORECE

        ";
        $params = [
                    ':CDTIPORECE'  => $cdTipoRece
                ];

        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function getPlanRequest($subsidiaryCode, $dtInicial, $dtFinal){
        $sql = "
               SELECT DISTINCT NRLANCESTQ FROM RETIRADA
                    WHERE CDFILIAL = :CDFILIAL
                        AND DTRETIRADA BETWEEN TO_DATE(:DATA_INI,'DD/MM/YYYY') AND TO_DATE(:DATA_FIM,'DD/MM/YYYY')

            ";
        $params = [
                ':CDFILIAL' => $subsidiaryCode,
                ':DATA_INI' => $dtInicial,
                ':DATA_FIM' => $dtFinal
                ];
        
        $sql = $this->connection->fetchAll($sql, $params);
        
        return $sql;
    }
    
    public function getSaleRequest($subsidiaryCode, $dtInicial, $dtFinal){
        $sql = "
               SELECT DISTINCT(IV.NRLANCESTQ) FROM VENDA V
                    JOIN ITEMVENDA IV
                    ON V.CDFILIAL = IV.CDFILIAL
                        AND V.CDCAIXA = IV.CDCAIXA
                        AND TO_DATE(V.DTENTRVENDA, 'DD/MM/YYYY') = TO_DATE(IV.DTHRPEDIDO, 'DD/MM/YYYY')
                WHERE V.CDFILIAL = :CDFILIAL
                    AND V.DTENTRVENDA BETWEEN TO_DATE(:DATA_INI, 'DD/MM/YYYY') AND TO_DATE(:DATA_FIM, 'DD/MM/YYYY')
                    AND IV.NRLANCESTQ IS NOT NULL
            ";
        $params = [
                ':CDFILIAL' => $subsidiaryCode,
                ':DATA_INI' => $dtInicial,
                ':DATA_FIM' => $dtFinal
                ];
        
        $sql = $this->connection->fetchAll($sql, $params);

        return $sql;
    }
    
    public function getMovEstoq($subsidiaryCode, $nrLancEstq){
        $sql = "
               SELECT IT.CDPRODUTO, P.NMPRODUTO, IT.NRLANCESTQ, IT.QTLANCTOEST, TO_DATE(IT.DTLANCMOVI, 'DD/MM/YYYY') AS DTLANCMOVI,
                       P.CDARVPROD, P.CDPRODINTE, P.SGUNIDADE, IV.CDPRODUTO AS PRODPAI, IV.NRSEQVENDA, V.NRNOTAFISCALCE,
                       NVL(RFL.QTSUBPRODLOC, RF.QTSUBPRODROT) AS QTDCOMP, IV.QTPRODVEND
                                FROM ITLANCTOEST IT
                    JOIN PRODUTO P
                        ON IT.CDPRODUTO = P.CDPRODUTO
                    LEFT JOIN ITEMVENDA IV
                        ON IV.CDFILIALEST  = IT.CDFILIMOVI
                         AND IV.NRLANCESTQ = IT.NRLANCESTQ
                         AND IV.CDPRODUTO  IN (SELECT NVL(NVL(RF.CDPRODUTO, RFL.CDPRODUTO), IT.CDPRODUTO) AS PRODPAI FROM PRODUTO P
                                                    LEFT JOIN ROTFABRICA RF
                                                        ON RF.CDSUBPRODUTO = P.CDPRODUTO
                                                    LEFT JOIN ROTFABLOCAL RFL
                                                        ON RFL.CDSUBPRODLOC = P.CDPRODUTO
                                                 WHERE P.CDPRODUTO = IT.CDPRODUTO)
                         AND IV.CDFILIAL   = IT.CDFILIMOVI
                    LEFT JOIN VENDA V
                        ON V.NRSEQVENDA = IV.NRSEQVENDA
                         AND V.CDFILIAL = IT.CDFILIMOVI
                    LEFT JOIN ROTFABRICA RF
                        ON IV.CDPRODUTO = RF.CDPRODUTO
                        AND IT.CDPRODUTO = RF.CDSUBPRODUTO
                    LEFT JOIN ROTFABLOCAL RFL
                        ON RFL.CDPRODUTO = IV.CDPRODUTO
                        AND RFL.CDSUBPRODLOC = IT.CDPRODUTO
                        AND RFL.CDFILIAL = IV.CDFILIAL
                    WHERE IT.CDFILIMOVI = :CDFILIAL
                        AND IT.NRLANCESTQ = :NRLANCESTQ
            ";
        $params = [
                ':CDFILIAL' => $subsidiaryCode,
                ':NRLANCESTQ' => $nrLancEstq
                ];
        
        $sql = $this->connection->fetchAll($sql, $params);

        return $sql;
    }
/*
    public function getMovEstoq($subsidiaryCode, $nrLancEstq){
        $sql = "
               SELECT IT.CDPRODUTO, P.NMPRODUTO, IT.NRLANCESTQ, IT.QTLANCTOEST, TO_DATE(IT.DTLANCMOVI, 'DD/MM/YYYY') AS DTLANCMOVI,
                       P.CDARVPROD, P.CDPRODINTE, P.SGUNIDADE, IV.NRSEQVENDA, V.NRNOTAFISCALCE
                FROM ITLANCTOEST IT
                    JOIN PRODUTO P
                        ON IT.CDPRODUTO = P.CDPRODUTO
                    LEFT JOIN ITEMVENDA IV
                        ON IV.CDFILIAL  = IT.CDFILIMOVI
                         AND IV.NRLANCESTQ = IT.NRLANCESTQ
                         AND IV.CDPRODUTO  = IT.CDPRODUTO
                    LEFT JOIN VENDA V
                        ON V.NRSEQVENDA = IV.NRSEQVENDA
                         AND V.CDFILIAL = IT.CDFILIMOVI
                    WHERE IT.CDFILIMOVI = :CDFILIAL
                        AND IT.NRLANCESTQ = :NRLANCESTQ
            ";
        $params = [
                ':CDFILIAL' => $subsidiaryCode,
                ':NRLANCESTQ' => $nrLancEstq
                ];
        
        $sql = $this->connection->fetchAll($sql, $params);

        return $sql;
    }
    */
    public function getDateMovEst($subsidiaryCode, $nrLancEstq){
        $sql = "
               SELECT TO_CHAR(DTLANCESTQ, 'DD-MM-YYYY') AS DTLANCESTQ FROM LANCTOESTOQ
                    WHERE CDFILIMOVI = :CDFILIAL
                    AND NRLANCESTQ = :NRLANCESTQ
            ";
        $params = [
                ':CDFILIAL' => $subsidiaryCode,
                ':NRLANCESTQ' => $nrLancEstq
                ];
        
        $sql = $this->connection->fetchAssoc($sql, $params);

        return $sql['DTLANCESTQ'];
    }
    
    public function getUnutilizedNotes($subsidiaryCode, $dtInicial, $dtFinal){
        $sql = "
                SELECT CDFILIAL, NRNOTAFISCALCE, DTENTRVENDA, CDSERIENFCE, IDTPAMBNFCE, IDSTATUSNFCE,
                       DTHRINUTNFCE, IDTPEMISINUNFCE, DSRAZAOINUTNFCE, DSOBSINUTNFCE, NRINSJURFILI
                FROM INUTILIZANFCE
                    WHERE CDFILIAL = :CDFILIAL
                        AND IDTPAMBNFCE = '1'
                        AND DTENTRVENDA BETWEEN TO_DATE(:DATA_INI,'dd/mm/yyyy') AND TO_DATE(:DATA_FIM,'dd/mm/yyyy')
                ORDER BY DTENTRVENDA 

        ";

        $params = [
            ':CDFILIAL' => $subsidiaryCode,
            ':DATA_INI' => $dtInicial,
            ':DATA_FIM' => $dtFinal
            ];
            
        
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getTypeItem($cdproduto){
        $sql = "
                SELECT NVL(P.CDTPITEMPROD, RI.CDTPITEMPROD) AS CDTPITEMPROD, TP.NMTPITEMPROD
                    FROM PRODUTO P
                    LEFT JOIN RELACPRODITGEN RI
                        ON RI.CDPRODUTO = P.CDPRODUTO
                        AND RI.CDLIVROMAG = (SELECT CDLIVROMAG FROM LIVROMAGNETICO
                                                WHERE IDPADRAOLIVRO = 'S')
                    LEFT JOIN TIPOITEMPROD TP
                        ON TP.CDTPITEMPROD = NVL(P.CDTPITEMPROD, RI.CDTPITEMPROD)
                        AND TP.CDLIVROMAG = (SELECT CDLIVROMAG FROM LIVROMAGNETICO
                                                WHERE IDPADRAOLIVRO = 'S')
                        WHERE P.CDPRODUTO = :CDPRODUTO

        ";

        $params = [
            ':CDPRODUTO' => $cdproduto
            ];
            
        return $this->connection->fetchAssoc($sql, $params);
    }
}